(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uarray.ml,v 1.29 2014/01/16 13:35:38 fclement Exp $ *)

(* More array operations. *)

let a_invalid_arg = Upervasives.invalid_arg "Uarray";;
let unknown_invalid_arg = Upervasives.invalid_arg "";;

let length a = Unatural.of_int (Array.length a);;
let unsafe_get = Array.unsafe_get;;
(*let unsafe_set = Array.unsafe_set;;*)
let make n = Array.make (Unatural.to_int n);;
let init n = Array.init (Unatural.to_int n);;

let is_empty a = length a = Unatural._0_;;

let check_nonempty fun_name a =
  if is_empty a then unknown_invalid_arg fun_name "empty array"
;;

let check_subarray fun_name a ofs len =
  if ofs < 0 || ofs + Unatural.to_int len > Unatural.to_int (length a) then
    a_invalid_arg fun_name "wrong subarray specification" 
;;

(* Filling arrays. *)

let unsafe_seti_sub f a ofs len =
  for i = ofs to pred (Unatural.to_int len) do a.(i) <- f i a.(i) done
;;

let seti_sub f a ofs len =
  check_subarray "seti_sub" a ofs len;
  unsafe_seti_sub f a ofs len
;;

let seti f a = unsafe_seti_sub f a 0 (length a);;

let unsafe_set_sub f = unsafe_seti_sub (fun _ ai -> f ai);;

let set_sub f a ofs len =
  check_subarray "set_sub" a ofs len;
  unsafe_set_sub f a ofs len
;;

let set f a = unsafe_set_sub f a 0 (length a);;

let list_of_list_array la = Array.to_list (Array.map Array.of_list la);;

let list_to_list_array al = Array.of_list (Ulist.map Array.to_list al);;

let of_list1 (x, l as l1) =
  let a1 = make (Ulist1.length l1) x in
  Ulist.iter_i (fun i x -> Array.unsafe_set a1 (succ i) x) l;
  a1
;;

(* Iterators. *)

let sub_iter_i f a ofs len =
  check_subarray "sub_iter" a ofs len;
  for i = ofs to ofs + pred (Unatural.to_int len) do f i (unsafe_get a i) done
;;

let sub_iter f = sub_iter_i (fun _ x -> f x);;

let sub_fold_left_i f accu a ofs len =
  check_subarray "sub_fold_left" a ofs len;
  let r = ref accu in
  for i = ofs to ofs + pred (Unatural.to_int len) do
    r := f !r i (unsafe_get a i)
  done;
  !r
;;

let sub_fold_left f = sub_fold_left_i (fun accu _ x -> f accu x);;

let fold_left_i f accu a = sub_fold_left_i f accu a 0 (length a);;

let fold_right_i f a x =
  let r = ref x in
  for i = 0 to Unatural.to_int (length a) - 1 do
    r := f i (unsafe_get a i) !r
  done;
  !r
;;

(* Iterators on two arrays. *)

let check_same_length fun_name a1 a2 =
  let len = length a1 in
  if length a2 <> len then
    a_invalid_arg fun_name "array arguments have different length" else
  len
;;

let iter_rails fa a =
  let len = check_same_length "iter_rails" fa a in
  for i = 0 to pred (Unatural.to_int len) do
    (unsafe_get fa i) (unsafe_get a i);
  done
;;

let iter2 f a1 a2 =
  let len = check_same_length "iter2" a1 a2 in
  for i = 0 to pred (Unatural.to_int len) do
    f (unsafe_get a1 i) (unsafe_get a2 i)
  done
;;

let map2 f a1 a2 =
  let len = check_same_length "map2" a1 a2 in
  init len (fun i -> f (unsafe_get a1 i) (unsafe_get a2 i))
;;

let iter_i2 f a1 a2 =
  let len = check_same_length "iter_i2" a1 a2 in
  for i = 0 to pred (Unatural.to_int len) do
    f i (unsafe_get a1 i) (unsafe_get a2 i)
  done
;;

let map_i2 f a1 a2 =
  let len = check_same_length "map_i2" a1 a2 in
  init len (fun i -> f i (unsafe_get a1 i) (unsafe_get a2 i))
;;

let fold_left2 f x a1 a2 =
  let len = check_same_length "fold_left2" a1 a2 in
  let r = ref x in
  for i = 0 to pred (Unatural.to_int len) do
    r := f !r (unsafe_get a1 i) (unsafe_get a2 i)
  done;
  !r
;;

let fold_right2 f a1 a2 x =
  let len = check_same_length "fold_right2" a1 a2 in
  let r = ref x in
  for i = pred (Unatural.to_int len) downto 0 do
    r := f (unsafe_get a1 i) (unsafe_get a2 i) !r
  done;
  !r
;;

let sub_iter_i2 f a1 ofs1 a2 ofs2 len =
  check_subarray "sub_iter2" a1 ofs1 len;
  check_subarray "sub_iter2" a2 ofs2 len;
  for i = 0 to pred (Unatural.to_int len) do
    f (ofs1 + i) (unsafe_get a1 (ofs1 + i))
      (ofs2 + i) (unsafe_get a2 (ofs2 + i))
  done
;;

let sub_iter2 f = sub_iter_i2 (fun _ x1 _ x2 -> f x1 x2);;

(* Iterators on more arrays. *)

let iter3 f a1 a2 a3 =
  let _len = check_same_length "iter3" a1 a2 in
  let len = check_same_length "iter3" a2 a3 in
  for i = 0 to pred (Unatural.to_int len) do
    f (unsafe_get a1 i) (unsafe_get a2 i) (unsafe_get a3 i)
  done
;;

let iter4 f a1 a2 a3 a4 =
  let _len = check_same_length "iter4" a1 a2 in
  let _len = check_same_length "iter4" a2 a3 in
  let len = check_same_length "iter4" a3 a4 in
  for i = 0 to pred (Unatural.to_int len) do
    f (unsafe_get a1 i) (unsafe_get a2 i) (unsafe_get a3 i) (unsafe_get a4 i)
  done
;;

let map3 f a1 a2 a3 =
  let _len = check_same_length "map3" a1 a2 in
  let len = check_same_length "map3" a2 a3 in
  init len (fun i -> f (unsafe_get a1 i) (unsafe_get a2 i) (unsafe_get a3 i))
;;

let map4 f a1 a2 a3 a4 =
  let _len = check_same_length "map4" a1 a2 in
  let _len = check_same_length "map4" a2 a3 in
  let len = check_same_length "map4" a3 a4 in
  init len
    (fun i ->
       f (unsafe_get a1 i) (unsafe_get a2 i)
         (unsafe_get a3 i) (unsafe_get a4 i))
;;

(* Iterators on list of arrays. *)

let maplist f al = Array.map f (list_to_list_array al);;

(* Array scanning. *)

let for_all p a =
  let len = Unatural.to_int (length a) in
  let rec loop i = i = len || p a.(i) && loop (succ i) in
  loop 0
;;

let exists p a =
  let len = Unatural.to_int (length a) in
  let rec loop i = i < len && (p a.(i) || loop (succ i)) in
  loop 0
;;

let unsafe_sub_find p a ofs len =
  let lim = ofs + Unatural.to_int len in
  let rec loop i =
    if i = lim then raise Not_found else
    if p a.(i) then a.(i) else loop (succ i) in
  loop ofs
;;

let sub_find p a ofs len =
  check_subarray "sub_find" a ofs len;
  unsafe_sub_find p a ofs len
;;

let find p a = unsafe_sub_find p a 0 (length a);;

let unsafe_sub_find_all p a ofs len =
  let lim = ofs + Unatural.to_int len in
  let rec loop accu i =
    if i = lim then accu else
    let accu = if p a.(i) then a.(i) :: accu else accu in
    loop accu (succ i) in
  loop [] ofs
;;

let sub_find_all p a ofs len =
  check_subarray "sub_find_all" a ofs len;
  unsafe_sub_find_all p a ofs len
;;

let find_all p a = unsafe_sub_find_all p a 0 (length a);;

let for_all2 p a1 a2 =
  let len = Unatural.to_int (check_same_length "for_all2" a1 a2) in
  let rec loop i = i = len || p a1.(i) a2.(i) && loop (succ i) in
  loop 0
;;

(* Sorted arrays. *)

let is_sorted comp a =
  let len = length a in
  if len = Unatural._0_ then true else
  let (sorted, _) =
    sub_fold_left
      (fun (b, aim1) ai -> (b && comp aim1 ai, ai))
      (true, a.(0)) a 1 (Unatural.pred len) in
  sorted
;;

let is_strictly_increasing a = is_sorted (fun x y -> x < y) a;;

let check_strictly_increasing fun_name a =
  if not (is_strictly_increasing a) then
    unknown_invalid_arg fun_name
      "elements of array are not strictly increasing"
;;

let sorted_array_find a x =
  let rec dicho l r =
    if l >= r then raise Not_found else
    let ip = Unatural.div (Unatural.add l r) Unatural._2_ in
    let p = Array.get a (Unatural.to_int ip) in
    if x == p then ip else
    if x < p then dicho l ip else
    dicho (Unatural.succ ip) r in
  dicho Unatural._0_ (length a)
;;
